/* 
 * Copyright 2014 Marc Nause <marc.nause@gmx.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see  http:// www.gnu.org/licenses/. 
 */
package de.audioattack.openlink;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;

import android.app.Activity;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.webkit.URLUtil;
import android.widget.Toast;

/**
 * Handles sharing of selected text.
 * 
 * @author Marc nause
 */
public class MenuActivity extends Activity {

    @Override
    protected final void onCreate(final Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // Get intent, action and MIME type
        final Intent intent = getIntent();
        final String action = intent.getAction();
        final String type = intent.getType();

        if (Intent.ACTION_SEND.equals(action) && type != null) {
            handleSendText(intent);
        } else {
            startActivity(new Intent(this, MainActivity.class));
        }

    }

    /**
     * Handles ACTION_SEND intent received via press on share item. Extracts
     * URls from input text (contained in Intent) and opens found URLs.
     * 
     * @param intent
     *            contains shared text
     */
    private void handleSendText(final Intent intent) {

        final String sharedText = intent.getStringExtra(Intent.EXTRA_TEXT);
        if (sharedText != null) {

            final String[] uris = getUris(sharedText);
            final List<String> errors = new ArrayList<String>();

            for (final String uri : uris) {
                try {

                    if (URLUtil.isValidUrl(uri)) {

                        startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse(uri)));
                    }
                } catch (Exception ex) {
                    errors.add(uri);
                }

            }

            if (!errors.isEmpty()) {
                final StringBuilder sb = new StringBuilder();
                final String lineSeparator = System.getProperty("line.separator");

                for (final String s : errors) {
                    if (sb.length() > 0) {
                        sb.append(lineSeparator);
                    }
                    sb.append(s);
                }

                Toast.makeText(this, this.getString(R.string.error_unable_to_open, sb.toString()), Toast.LENGTH_LONG)
                        .show();
            } else if (uris.length == 0) {
                Toast.makeText(this, this.getString(R.string.error_no_url_found), Toast.LENGTH_LONG).show();
            }

            this.finish();
        }
    }

    /**
     * Retrieves all Strings which look remotely like URLs from a text.
     * 
     * @param sharedText
     *            text to scan for URLs.
     * @return potential URLs
     */
    private String[] getUris(final String sharedText) {

        final Collection<String> result = new HashSet<String>();

        if (sharedText != null) {

            final String[] array = sharedText.split("\\p{White_Space}");

            for (final String s : array) {

                if (s.length() != 0) {
                    if (s.matches(".+://.+")) {
                        result.add(s);
                    } else if (s.matches(".+\\..+")) {
                        result.add("http://" + s);
                    }
                }
            }
        }

        return result.toArray(new String[0]);
    }

}
